@@ -1,4 +1,5 @@
 import numbers
+import warnings
 from collections.abc import Iterable
 from copy import deepcopy
 import numpy as np
@@ -43,6 +44,7 @@ def process_noise(pulses, noise_list, dims, t1=None, t2=None,
     noisy_pulses: list of :class:`.Pulse`
         The noisy pulses, including the system noise.
     """
+    noise_list = noise_list.copy()
     noisy_pulses = deepcopy(pulses)
     systematic_noise = Pulse(None, None, label="systematic_noise")
 
@@ -70,13 +72,12 @@ class Noise(object):
     The noise object can be added to :class:`.device.Processor` and
     contributes to evolution.
     """
-    def __init__(self):
-        pass
-
-    def get_noisy_dynamics(self, dims, pulses, systematic_noise):
+    def get_noisy_pulses(self, dims=None, pulses=None, systematic_noise=None):
         """
         Return the input pulses list with noise added and
-        the pulse independent noise in a dummy Pulse object.
+        the pulse independent noise in a dummy :class:`.Pulse` object.
+        This is a template method, a method with the same name and signatures
+        needs to be defined in the subclasses.
 
         Parameters
         ----------
@@ -85,7 +86,7 @@ class Noise(object):
             [2,2...,2] for qubits system.
 
         pulses : list of :class:`.Pulse`
-            The input pulses, on which the noise object is to be applied.
+            The input pulses. The noise will be added to pulses in this list.
 
         systematic_noise : :class:`.Pulse`
             The dummy pulse with no ideal control element.
@@ -96,18 +97,26 @@ class Noise(object):
             Noisy pulses.
 
         systematic_noise : :class:`.Pulse`
-            The dummy pulse representing pulse independent noise.
+            The dummy pulse representing pulse-independent noise.
         """
+        get_noisy_dynamics = getattr(self, "get_noisy_dynamics", None)
+        if get_noisy_dynamics is not None:
+            warnings.warn(
+                "Using get_noisy_dynamics as the hook function for custom "
+                "noise will be deprecated, "
+                "please use get_noisy_pulses instead.",
+                PendingDeprecationWarning)
+            return self.get_noisy_dynamics(dims, pulses, systematic_noise)
         raise NotImplementedError(
             "Subclass error needs a method"
-            "`get_noisy_dynamics` to process the noise.")
+            "`get_noisy_pulses` to process the noise.")
 
     def _apply_noise(self, pulses=None, systematic_noise=None, dims=None):
         """
         For backward compatibility, in case the method has no return value
         or only return the pulse.
         """
-        result = self.get_noisy_dynamics(
+        result = self.get_noisy_pulses(
             pulses=pulses, systematic_noise=systematic_noise, dims=dims)
         if result is None:  # in-place change
             pass
@@ -118,7 +127,7 @@ class Noise(object):
             pulses = result
         else:
             raise TypeError(
-                "Returned value of get_noisy_dynamics not understood.")
+                "Returned value of get_noisy_pulses not understood.")
         return pulses, systematic_noise
 
 
@@ -172,8 +181,29 @@ class DecoherenceNoise(Noise):
                     "thus cannot be applied to all qubits")
         self.all_qubits = all_qubits
 
-    def get_noisy_dynamics(
+    def get_noisy_pulses(
             self, dims=None, pulses=None, systematic_noise=None):
+        """
+        Return the input pulses list with noise added and
+        the pulse independent noise in a dummy :class:`.Pulse` object.
+
+        Parameters
+        ----------
+        dims: list, optional
+            The dimension of the components system, the default value is
+            [2, 2, ..., 2] for a system of qubits.
+        pulses : list of :class:`.Pulse`
+            The input pulses. The noise will be added to pulses in this list.
+        systematic_noise : :class:`.Pulse`
+            The dummy pulse with no ideal control element.
+
+        Returns
+        -------
+        noisy_pulses: list of :class:`.Pulse`
+            Noisy pulses.
+        systematic_noise : :class:`.Pulse`
+            The dummy pulse representing pulse-independent noise.
+        """
         if systematic_noise is None:
             systematic_noise = Pulse(None, None, label="system")
         N = len(dims)
@@ -250,8 +280,29 @@ class RelaxationNoise(Noise):
                 "either the length is not equal to the number of qubits, "
                 "or T is not a positive number.".format(T))
 
-    def get_noisy_dynamics(
+    def get_noisy_pulses(
             self, dims=None, pulses=None, systematic_noise=None):
+        """
+        Return the input pulses list with noise added and
+        the pulse independent noise in a dummy :class:`.Pulse` object.
+
+        Parameters
+        ----------
+        dims: list, optional
+            The dimension of the components system, the default value is
+            [2,2...,2] for qubits system.
+        pulses : list of :class:`.Pulse`
+            The input pulses. The noise will be added to pulses in this list.
+        systematic_noise : :class:`.Pulse`
+            The dummy pulse with no ideal control element.
+
+        Returns
+        -------
+        noisy_pulses: list of :class:`.Pulse`
+            Noisy pulses.
+        systematic_noise : :class:`.Pulse`
+            The dummy pulse representing pulse-independent noise.
+        """
         if systematic_noise is None:
             systematic_noise = Pulse(None, None, label="system")
         N = len(dims)
@@ -319,7 +370,7 @@ class ControlAmpNoise(Noise):
         self.tlist = tlist
         self.indices = indices
 
-    def get_noisy_dynamics(
+    def get_noisy_pulses(
             self, dims=None, pulses=None, systematic_noise=None):
         if pulses is None:
             pulses = []
@@ -389,8 +440,29 @@ class RandomNoise(ControlAmpNoise):
         self.dt = dt
         self.indices = indices
 
-    def get_noisy_dynamics(
+    def get_noisy_pulses(
             self, dims=None, pulses=None, systematic_noise=None):
+        """
+        Return the input pulses list with noise added and
+        the pulse independent noise in a dummy :class:`.Pulse` object.
+
+        Parameters
+        ----------
+        dims: list, optional
+            The dimension of the components system, the default value is
+            [2,2...,2] for qubits system.
+        pulses : list of :class:`.Pulse`
+            The input pulses. The noise will be added to pulses in this list.
+        systematic_noise : :class:`.Pulse`
+            The dummy pulse with no ideal control element.
+
+        Returns
+        -------
+        noisy_pulses: list of :class:`.Pulse`
+            Noisy pulses.
+        systematic_noise : :class:`.Pulse`
+            The dummy pulse representing pulse-independent noise.
+        """
         if pulses is None:
             pulses = []
         if self.indices is None:
@@ -432,8 +504,29 @@ class ZZCrossTalk(Noise):
             self, params):
         self.params = params
 
-    def get_noisy_dynamics(
+    def get_noisy_pulses(
             self, dims=None, pulses=None, systematic_noise=None):
+        """
+        Return the input pulses list with noise added and
+        the pulse independent noise in a dummy :class:`.Pulse` object.
+
+        Parameters
+        ----------
+        dims: list, optional
+            The dimension of the components system, the default value is
+            [2,2...,2] for qubits system.
+        pulses : list of :class:`.Pulse`
+            The input pulses. The noise will be added to pulses in this list.
+        systematic_noise : :class:`.Pulse`
+            The dummy pulse with no ideal control element.
+
+        Returns
+        -------
+        noisy_pulses: list of :class:`.Pulse`
+            Noisy pulses.
+        systematic_noise : :class:`.Pulse`
+            The dummy pulse representing pulse-independent noise.
+        """
         J = self.params["J"]
         wr_dr = self.params["wr_dressed"]
         wr = self.params["wr"]
